#!/bin/bash

KCL="amd/amdkcl"
INC="include"
SRC="amd/dkms"

KERNELVER=$1
KERNELVER_BASE=${KERNELVER%%-*}
SRCTREE=/lib/modules/$KERNELVER

if [ -L $SRCTREE/source ]; then
	BLDTREE="$SRCTREE/build"
	SRCTREE="$SRCTREE/source"
else
	SRCTREE="$SRCTREE/build"
	BLDTREE="$SRCTREE"
fi

SRCARCH=$(uname -m | sed -e "s/i.86/x86/" -e "s/x86_64/x86/" \
        -e "s/sun4u/sparc64/" -e "s/arm.*/arm/" -e "s/sa110/arm/" \
        -e "s/s390x/s390/" -e "s/parisc64/parisc/" \
        -e "s/ppc.*/powerpc/" -e "s/mips.*/mips/" \
        -e "s/sh[234].*/sh/" -e "s/aarch64.*/arm64/")

version_lt () {
    newest=$((echo "$KERNELVER_BASE"; echo "$1") | sort -V | tail -n1)
    [ "$KERNELVER_BASE" != "$newest" ]
}

version_ge () {
    newest=$((echo "$KERNELVER_BASE"; echo "$1") | sort -V | tail -n1)
    [ "$KERNELVER_BASE" = "$newest" ]
}

version_gt () {
    oldest=$((echo "$KERNELVER_BASE"; echo "$1") | sort -V | head -n1)
    [ "$KERNELVER_BASE" != "$oldest" ]
}

version_le () {
    oldest=$((echo "$KERNELVER_BASE"; echo "$1") | sort -V | head -n1)
    [ "$KERNELVER_BASE" = "$oldest" ]
}

source $KCL/symbols
source $KCL/files

echo '// auto generated by DKMS pre-build.sh' > $KCL/symbols.c
for sym in $SYMS; do
	awk -v sym=$sym '/\<sym\>/ {
		print "void *_kcl_" $3 " = (void *)0x" $1 ";"
	}' /boot/System.map-$KERNELVER >>$KCL/symbols.c
done

sed -i -e '/DEFINE_WD_CLASS(reservation_ww_class)/,/EXPORT_SYMBOL(reservation_seqcount_string)/d' \
       -e 's/linux\/sched\/mm\.h/kcl\/header\/kcl_sched_mm_h\.h/' \
       -e '/dma_resv_lockdep/,/subsys_initcall/d' $KCL/dma-buf/dma-resv.c
sed -i -e '/define _LINUX_RESERVATION_H/a #include <kcl/kcl_dma-resv.h>' \
       -e '/extern struct ww_class reservation_ww_class/,/extern const char reservation_seqcount_string/d' \
       -e '/struct dma_resv {/, /}/d' $INC/linux/dma-resv.h

while read from to; do
	[[ -z "$from" ]] && continue
	file=${from//\//\\/}
	file=${file//\./\\.}
	list=$(grep -rl "$file" amd/* scheduler/* ttm/* include/* | \
		sed '/^include\/kcl/d; /^amd\/dkms/d; /^amd\/amdkcl/d' | sort -u)
	for file in $list; do
		sed -i "s|$from|$to|" $file
	done
done < $SRC/headers

for file in $FILES; do
	awk -F'[()]' '/EXPORT_SYMBOL/ {
		print "#define "$2" amd"$2" //"$0
	}' $file | sort -u >>$INC/rename_symbol.h
done

# rename CONFIG_xxx to CONFIG_xxx_AMDKCL
# otherwise kernel config would override dkms package config
AMDGPU_CONFIG=$(find -name Kconfig -exec grep -h '^config' {} + | sed 's/ /_/' | tr 'a-z' 'A-Z')
TTM_CONFIG=$(awk '/CONFIG_DRM/{gsub(".*\\(CONFIG_DRM","CONFIG_DRM");gsub("\\).*","");print $0}' ttm/Makefile)
SCHED_CONFIG=$(awk '/CONFIG_DRM/{gsub(".*\\(CONFIG_DRM","CONFIG_DRM");gsub("\\).*","");print $0}' scheduler/Makefile)
for config in $AMDGPU_CONFIG $TTM_CONFIG $SCHED_CONFIG; do
	for file in $(grep -rl $config ./); do
		sed -i "s/\<$config\>/&_AMDKCL/" $file
	done
	sed -i "/${config}$/s/$/_AMDKCL/" amd/dkms/Makefile
done

export KERNELVER
(cd $SRC && CPPFLAGS="-I$SRCTREE/arch/$SRCARCH/include \
	-I$BLDTREE/arch/$SRCARCH/include/generated \
	-I$SRCTREE/include \
	-I$BLDTREE/include \
	-I$SRCTREE/include/uapi \
	-include $SRCTREE/include/linux/kconfig.h" \
	./configure)

FW_DIR="/lib/firmware/$KERNELVER"
mkdir -p $FW_DIR
cp -ar /usr/src/amdgpu-3.10-29/firmware/amdgpu $FW_DIR
